function push(page) {
    log("Pushing " + page.name + " to the stack.");
    _pageStack.push(page);
    logTheStack();
}

function pop() {
    return _pageStack.pop();
}

function pull(page) {
    log("Pulling " + page.name + " from the stack");
    var index = find(page);
    var p;
    log("Found index = " + index);
    index > -1 ? p = _pageStack.splice(index, 1) : p[0] = null;
    logTheStack();
    return p[0];
}

function open(page) {
    log("Opening page: " + page.name + " | type: " + page.type);
    var c, v;
    var p = findByName(page.name);
    if (null !== p) {
        log(p.name + " has already been opened. Promoting page.");
        promote(p);
        return;
    }
    if ("controller" == page.type) {
        c = Alloy.createController(page.name);
        v = c.getView();
        push(page);
        v.open(fadeInAnimation);
    } else if ("web" == page.type) {
        log("OPENING a BROWSERVIEW : {name : " + page.name + ", link: " + page.url + ", title: " + page.title + "}");
        c = Alloy.createController("browserview", {
            name: page.name,
            link: page.url,
            title: page.title
        });
        v = c.getView();
        push(page);
        v.open(fadeInAnimation);
    } else {
        log("Attempted to open an unrecognized page type: " + page.type);
        alert("You tried to open an unrecgonized page type: " + page.type);
    }
    logTheStack();
}

function close(page) {
    page = pull(page);
    var view = Alloy.Globals.activeWindows[page.name];
    view.close(fadeOutAnimation);
    logTheStack();
}

function toggle(page) {
    var index = find(page);
    find(page) > -1 ? index == _pageStack.length - 1 ? close(page) : promote(page) : open(page);
}

function closeAll() {
    Titanium.API.info("PAGEUTILS::closeAll: Attempting to close all pages");
    logTheStack();
    while (_pageStack.length > 0) {
        log("PAGEUTILS::closeAll: Attempting to close " + _pageStack.last().name);
        close(_pageStack.last());
    }
}

function find(page) {
    log("Searching for " + page.name + "...");
    for (var i = 0; i < _pageStack.length; i++) if (_pageStack[i].name == page.name) {
        log("Found " + page.name + " at " + i + " in find()");
        return i;
    }
    log(page.name + " NOT FOUND in find()");
    return -1;
}

function findByName(name) {
    log("Searching for " + name + "...");
    var index = find({
        name: name
    });
    if (index > -1) {
        log("Found " + name + " at " + index + " in findByName()");
        return _pageStack[index];
    }
    log(name + " NOT FOUND in findByName()");
    logTheStack();
    return null;
}

function promote(page) {
    log("Promoting " + page.name);
    var index = find(page);
    if (index > -1) {
        close(page);
        log("Closed " + page.name + " during promotion process...");
        open(page);
        log("Opened " + page.name + " during promotion process...");
        log("After promoting " + page.name + " to the stack during promotion");
    }
}

function promoteByName(name) {
    promote({
        name: name
    });
}

function logTheStack() {
    var str = "DEBUG: _pageStack = {";
    var names = [];
    for (var i = 0; i < _pageStack.length; i++) names.push(_pageStack[i].name);
    str += names.toString();
    str += "}";
    log(str);
}

function log(msg) {
    if (Alloy.Globals.logLevel > 1) {
        Titanium.API.info(msg);
        TF.remoteLog(msg);
    }
}

var Alloy = require("alloy"), _ = require("alloy/underscore")._, Backbone = require("alloy/backbone");

var _pageStack = [];

var fadeOutAnimation = Titanium.UI.createAnimation({
    opacity: "0",
    duration: 200,
    curve: Titanium.UI.ANIMATION_CURVE_EASE_IN
});

var fadeInAnimation = Titanium.UI.createAnimation({
    opacity: "1",
    duration: 200,
    curve: Titanium.UI.ANIMATION_CURVE_EASE_IN
});

Array.prototype.last || (Array.prototype.last = function() {
    return this[this.length - 1];
});

exports.push = push;

exports.pop = pop;

exports.pull = pull;

exports.open = open;

exports.close = close;

exports.closeAll = closeAll;

exports.find = find;

exports.findByName = findByName;

exports.promote = promote;

exports.toggle = toggle;

exports.promoteByName = promoteByName;

exports.log = log;